export declare function parseTimeout(raw: string | undefined, fallback: number): number;
export declare const LIST_TIMEOUT_MS: number;
export declare function resolveCallTimeout(override?: number): number;
export declare function withTimeout<T>(promise: Promise<T>, timeoutMs: number): Promise<T>;
export declare function consumeTimeoutFlag(args: string[], index: number, options?: {
    flagName?: string;
    missingValueMessage?: string;
}): number;
//# sourceMappingURL=timeouts.d.ts.map