export declare function getDaemonMetadataPath(configKey: string): string;
export declare function getDaemonSocketPath(configKey: string): string;
export declare function getDaemonLogPath(configKey: string): string;
export declare function getDaemonDir(): string;
//# sourceMappingURL=paths.d.ts.map