export declare function normalizeHttpUrlCandidate(value?: string): string | undefined;
export declare function looksLikeHttpUrl(value?: string): boolean;
export declare function splitHttpToolSelector(input: string): {
    baseUrl: string;
    tool: string;
} | null;
export declare function normalizeHttpUrl(value: string | URL): string | undefined;
export declare function extractHttpServerTarget(value: string): string | undefined;
//# sourceMappingURL=http-utils.d.ts.map