export type StdioLogMode = 'auto' | 'always' | 'silent';
export declare function getStdioLogMode(): StdioLogMode;
export declare function setStdioLogMode(mode: StdioLogMode): StdioLogMode;
export declare function evaluateStdioLogPolicy(mode: StdioLogMode, hasStderr: boolean, exitCode: number | null | undefined): boolean;
//# sourceMappingURL=sdk-patches.d.ts.map