import type { CommandSpec, RawLifecycle, ServerDefinition, ServerLifecycle } from './config-schema.js';
declare const DEFAULT_KEEP_ALIVE: Set<string>;
export declare function resolveLifecycle(name: string, rawLifecycle: RawLifecycle | undefined, command: CommandSpec): ServerLifecycle | undefined;
export declare function canonicalKeepAliveName(command: CommandSpec): string | undefined;
export declare function isKeepAliveServer(definition: ServerDefinition | undefined): boolean;
export declare function keepAliveIdleTimeout(definition: ServerDefinition): number | undefined;
export { DEFAULT_KEEP_ALIVE };
//# sourceMappingURL=lifecycle.d.ts.map