import type { EphemeralServerSpec } from './adhoc-server.js';
interface ExtractOptions {
    allowPersist?: boolean;
}
export declare function extractEphemeralServerFlags(args: string[], options?: ExtractOptions): EphemeralServerSpec | undefined;
export {};
//# sourceMappingURL=ephemeral-flags.d.ts.map