import type { Transport } from '@modelcontextprotocol/sdk/shared/transport.js';
import type { Logger } from './logging.js';
export declare function closeTransportAndWait(logger: Logger, transport: Transport & {
    close(): Promise<void>;
}): Promise<void>;
declare function listDescendantPids(rootPid: number): Promise<number[]>;
export declare const __testHooks: {
    listDescendantPids: typeof listDescendantPids;
};
export {};
//# sourceMappingURL=runtime-process-utils.d.ts.map